@@ -3,11 +3,12 @@
 
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
-    Copyright 2009-2010, Alan Calvert
+    Copyright 2009-2011, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of version 2 of the GNU General Public
-    License as published by the Free Software Foundation.
+    it and/or modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either version 2 of
+    the License, or (at your option) any later version.
 
     yoshimi is distributed in the hope that it will be useful, but WITHOUT ANY
     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -18,7 +19,7 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is a derivative of a ZynAddSubFX original, modified October 2010
+    This file is a derivative of a ZynAddSubFX original, modified March 2011
 */
 
 #ifndef DYNAMICFILTER_H
@@ -34,19 +35,16 @@ class DynamicFilter : public Effect
         DynamicFilter(bool insertion_, float *efxoutl_, float *efxoutr_);
         ~DynamicFilter();
         void out(float *smpsl, float *smpsr);
-
         void setpreset(unsigned char npreset);
         void changepar(int npar, unsigned char value);
         unsigned char getpar(int npar);
         void cleanup(void);
-
     //	void setdryonly();
 
     private:
         // Parametrii DynamicFilter
         EffectLFO lfo; // lfo-ul DynamicFilter
         unsigned char Pvolume;
-        unsigned char Ppanning;
         unsigned char Pdepth;
         unsigned char Pampsns;
         unsigned char Pampsnsinv; // if the filter freq is lowered if the input amplitude rises
@@ -54,13 +52,12 @@ class DynamicFilter : public Effect
 
         // Parameter Control
         void setvolume(unsigned char Pvolume_);
-        void setpanning(unsigned char Ppanning_);
         void setdepth(unsigned char Pdepth_);
         void setampsns(unsigned char Pampsns_);
         void reinitfilter(void);
 
         // Internal Values
-        float panning, depth, ampsns, ampsmooth;
+        float depth, ampsns, ampsmooth;
 
         Filter *filterl, *filterr;
         float ms1, ms2, ms3, ms4; // mean squares
