@@ -5,6 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
     Copyright 2013, Nikita Zlobin
+    Copyright 2014, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -20,7 +21,7 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, modified April 2011
+    This file is derivative of ZynAddSubFX original code, modified August 2014
 */
 
 #include <iostream>
@@ -108,6 +109,7 @@ Config::Config() :
     GzipCompression(3),
     Interpolation(0),
     CheckPADsynth(1),
+    EnableProgChange(0), // default will be inverted
     rtprio(50),
     midi_bank_C(0),
     midi_upper_voice_C(128),
@@ -174,7 +176,7 @@ bool Config::Setup(int argc, char **argv)
             break;
     }
     if (!midiDevice.size())
-        midiDevice = "default";
+        midiDevice = "";
     loadCmdArgs(argc, argv);
     if (restoreState)
     {
@@ -361,6 +363,7 @@ bool Config::extractConfigData(XMLwrapper *xml)
     currentBankDir = xml->getparstr("bank_current");
     Interpolation = xml->getpar("interpolation", Interpolation, 0, 1);
     CheckPADsynth = xml->getpar("check_pad_synth", CheckPADsynth, 0, 1);
+    EnableProgChange = 1 - xml->getpar("ignore_program_change", EnableProgChange, 0, 1); // inverted for Zyn compatibility
     VirKeybLayout = xml->getpar("virtual_keyboard_layout", VirKeybLayout, 0, 10);
 
     // get bank dirs
@@ -490,6 +493,7 @@ void Config::addConfigXML(XMLwrapper *xmltree)
 
     xmltree->addpar("gzip_compression", GzipCompression);
     xmltree->addpar("check_pad_synth", CheckPADsynth);
+    xmltree->addpar("ignore_program_change", (1 - EnableProgChange));
     xmltree->addparstr("bank_current", currentBankDir);
     xmltree->addpar("virtual_keyboard_layout", VirKeybLayout);
 
@@ -657,6 +661,8 @@ void Config::StartupReport(void)
         default:
             report += "nada";
     }
+    if (!midiDevice.size())
+        midiDevice = "default";
     report += (" -> '" + midiDevice + "'");
     Log(report);
     Log("Oscilsize: " + asString(synth->oscilsize));
@@ -674,7 +680,7 @@ void Config::setRtprio(int prio)
 
 // general thread start service
 bool Config::startThread(pthread_t *pth, void *(*thread_fn)(void*), void *arg,
-                         bool schedfifo, bool lowprio)
+                         bool schedfifo, char priodec)
 {
     pthread_attr_t attr;
     int chk;
@@ -706,8 +712,8 @@ bool Config::startThread(pthread_t *pth, void *(*thread_fn)(void*), void *arg,
                     }
                     sched_param prio_params;
                     int prio = rtprio;
-                    if (lowprio)
-                        --prio;
+                    if (priodec)
+                        prio -= priodec;
                     prio_params.sched_priority = (prio > 0) ? prio : 0;
                     if ((chk = pthread_attr_setschedparam(&attr, &prio_params)))
                     {
@@ -953,7 +959,7 @@ static error_t parse_cmds (int key, char *arg, struct argp_state *state)
             settings->midiEngine = alsa_midi;
             if (arg)
                 settings->midiDevice = string(arg);
-                  break;
+            break;
         case 'i':
             settings->showGui = false;
             settings->showConsole = false;
