@@ -4,6 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
+    Copyright 2014, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -19,7 +20,7 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, modified April 2011
+    This file is derivative of ZynAddSubFX original code, modified August 2014
 */
 
 #ifndef CONFIG_H
@@ -71,7 +72,7 @@ class Config : public MiscFuncs
         void signalCheck(void);
         void setRtprio(int prio);
         bool startThread(pthread_t *pth, void *(*thread_fn)(void*), void *arg,
-                         bool schedfifo, bool lowprio);
+                         bool schedfifo, char lowprio);
 
         string addParamHistory(string file);
         string historyFilename(int index);
@@ -118,11 +119,11 @@ class Config : public MiscFuncs
         string        currentBankDir;
         string        presetsDirlist[MAX_BANK_ROOT_DIRS];
         int           CheckPADsynth;
+        int           EnableProgChange;
         int           rtprio;
         unsigned int  midi_bank_C;
         unsigned int  midi_upper_voice_C;
         int           enable_part_on_voice_load;
-        char          AudioSend[NUM_MIDI_PARTS];
 
         deque<HistoryListItem> ParamsHistory;
         deque<HistoryListItem>::iterator itx;
