@@ -4,6 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
+    Copyright 2014, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -19,7 +20,7 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, modified March 2011
+    This file is derivative of ZynAddSubFX original code, modified August 2014
 */
 
 #ifndef SYNTHENGINE_H
@@ -27,6 +28,7 @@
 
 #include <limits.h>
 #include <cstdlib>
+#include <jack/ringbuffer.h>
 
 using namespace std;
 
@@ -50,7 +52,6 @@ class SynthEngine : private SynthHelper, MiscFuncs
         ~SynthEngine();
         bool Init(unsigned int audiosrate, int audiobufsize);
         bool actionLock(lockset request);
-        bool vupeakLock(lockset request);
 
         bool saveXML(string filename);
         void add2XML(XMLwrapper *xml);
@@ -93,6 +94,7 @@ class SynthEngine : private SynthHelper, MiscFuncs
         float halfoscilsize_f;
 
         unsigned char Pvolume;
+        int           Paudiodest;
         int           Pkeyshift;
         unsigned char Psysefxvol[NUM_SYS_EFX][NUM_MIDI_PARTS];
         unsigned char Psysefxsend[NUM_SYS_EFX][NUM_SYS_EFX];
@@ -102,6 +104,7 @@ class SynthEngine : private SynthHelper, MiscFuncs
         void setPkeyshift(int Pkeyshift_);
         void setPsysefxvol(int Ppart, int Pefx, char Pvol);
         void setPsysefxsend(int Pefxfrom, int Pefxto, char Pvol);
+        void setPaudiodest(int value);
 
         // effects
         EffectMgr *sysefx[NUM_SYS_EFX]; // system
@@ -112,24 +115,27 @@ class SynthEngine : private SynthHelper, MiscFuncs
 
         // peaks for part VU-meters
         float vuoutpeakpart[NUM_MIDI_PARTS];
-        unsigned char fakepeakpart[NUM_MIDI_PARTS]; // this is used to compute the
-                                                    // "peak" when the part is disabled
+
         // others ...
         Controller *ctl;
         Microtonal microtonal;
         Bank bank;
         FFTwrapper *fft;
 
-        // peaks for VU-meters
-        void vuresetpeaks(void);
-        float vuOutPeakL;
-        float vuOutPeakR;
-        float vuMaxOutPeakL;
-        float vuMaxOutPeakR;
-        float vuRmsPeakL;
-        float vuRmsPeakR;
-        bool vuClippedL;
-        bool vuClippedR;
+        // peaks for VU-meters        
+        union VUtransfer{
+            struct{
+                float vuOutPeakL;
+                float vuOutPeakR;
+                float vuRmsPeakL;
+                float vuRmsPeakR;
+                float parts[NUM_MIDI_PARTS];
+            } values;
+            char bytes [sizeof(values)];
+        };
+        union VUtransfer VUpeak, VUdata;
+        
+        bool fetchMeterData(VUtransfer *VUdata);
 
 
     private:
@@ -141,22 +147,13 @@ class SynthEngine : private SynthHelper, MiscFuncs
         float *tmpmixr; // which are sent to system effect
         int keyshift;
 
-        float vuoutpeakl;
-        float vuoutpeakr;
-        float vumaxoutpeakl;
-        float vumaxoutpeakr;
-        float vurmspeakl;
-        float vurmspeakr;
-        bool clippedL;
-        bool clippedR;
-
         pthread_mutex_t  processMutex;
         pthread_mutex_t *processLock;
-        pthread_mutex_t  meterMutex;
-        pthread_mutex_t *meterLock;
 
+        jack_ringbuffer_t *vuringbuf;
+        
         XMLwrapper *stateXMLtree;
-
+        
         static char random_state[];
         static struct random_data random_buf;
         int32_t random_result;
