@@ -2,6 +2,7 @@
     AlsaEngine.cpp
 
     Copyright 2009-2011, Alan Calvert
+    Copyright 2014, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -15,6 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
+    Last modified August 2014
 */
 
 #include "Misc/Config.h"
@@ -62,12 +64,11 @@ bail_out:
 bool AlsaEngine::openMidi(void)
 {
     midi.device = Runtime.midiDevice;
-    if (midi.device.empty())
-        midi.device = "default";
     const char* port_name = "input";
-    if (snd_seq_open(&midi.handle, midi.device.c_str(), SND_SEQ_OPEN_INPUT, 0))
+    int port_num;
+    if (snd_seq_open(&midi.handle, "default", SND_SEQ_OPEN_INPUT, 0) != 0)
     {
-        Runtime.Log("Error, failed to open alsa midi device: " + midi.device);
+        Runtime.Log("Error, failed to open alsa midi");
         goto bail_out;
     }
     snd_seq_client_info_t *seq_info;
@@ -86,16 +87,32 @@ bool AlsaEngine::openMidi(void)
     snd_seq_client_info_event_filter_add(seq_info, SND_SEQ_EVENT_PORT_UNSUBSCRIBED);
     if (0 > snd_seq_set_client_info(midi.handle, seq_info))
         Runtime.Log("Failed to set midi event filtering");
+    
     snd_seq_set_client_name(midi.handle, midiClientName().c_str());
-
-    if (0 > snd_seq_create_simple_port(midi.handle, port_name,
+    
+    port_num = snd_seq_create_simple_port(midi.handle, port_name,
                                        SND_SEQ_PORT_CAP_WRITE
                                        | SND_SEQ_PORT_CAP_SUBS_WRITE,
-                                       SND_SEQ_PORT_TYPE_SYNTH))
+                                       SND_SEQ_PORT_TYPE_SYNTH);
+    if (port_num < 0)
     {
         Runtime.Log("Error, failed to acquire alsa midi port");
         goto bail_out;
     }
+    if (!midi.device.empty())
+    {
+        bool midiSource = false;
+        if (snd_seq_parse_address(midi.handle,&midi.addr,midi.device.c_str()) == 0)
+        {
+            midiSource = (snd_seq_connect_from(midi.handle, port_num, midi.addr.client, midi.addr.port) == 0);
+        }
+        if (!midiSource)
+        {
+            Runtime.Log("Didn't find alsa MIDI source '" + midi.device + "'");
+            Runtime.midiDevice = "";
+        }
+        
+    }
     return true;
 
 bail_out:
@@ -379,13 +396,13 @@ bool AlsaEngine::xrunRecover(void)
 bool AlsaEngine::Start(void)
 {
     if (NULL != midi.handle && !Runtime.startThread(&midi.pThread, _MidiThread,
-                                                    this, true, true))
+                                                    this, true, 1))
     {
         Runtime.Log("Failed to start Alsa midi thread");
         goto bail_out;
     }
     if (NULL != audio.handle && !Runtime.startThread(&audio.pThread, _AudioThread,
-                                                     this, true, false))
+                                                     this, true, 0))
     {
         Runtime.Log(" Failed to start Alsa audio thread");
         goto bail_out;
